﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace LiveCoding0827
{
    /// <summary>
    /// キーフレームを使わないアニメーションをコードから簡単に作成できるクラス
    /// </summary>
    /// <typeparam name="Type">アニメーションクラス</typeparam>
    /// <typeparam name="Easing">イージング関数</typeparam>
    /// <typeparam name="ValueType">アニメーションさせる値の型</typeparam>
    class SimpleAnimation<Type, ValueType>
        where Type : Timeline, new()
        where ValueType : struct
    {
        #region Private
        private Storyboard storyboard_;
        private Timeline animation_;

        private T GetValueUsingRefrection<T>(object obj, string name)
        {
            var Property = obj.GetType().GetProperty(name);
            if (Property == null) return default(T);
            return (T)Property.GetValue(obj, null);
        }

        private void SetValueUsingRefrection<T>(object obj, string name, T value)
        {
            var Property = obj.GetType().GetProperty(name);
            if (Property == null) return;
            Property.SetValue(obj, value, null);
        }
        #endregion

        public SimpleAnimation()
        {
            storyboard_ = new Storyboard();
            animation_ = new Type();
        }

        public ValueType? From { get { return GetValueUsingRefrection<ValueType?>(animation_, "From"); } set { SetValueUsingRefrection(animation_, "From", value); } }
        public ValueType? To { get { return GetValueUsingRefrection<ValueType?>(animation_, "To"); } set { SetValueUsingRefrection(animation_, "To", value); } }
        public Duration Duration { get { return animation_.Duration; } set { animation_.Duration = value; } }

        /// <summary>
        /// アニメーションを開始させる
        /// </summary>
        /// <param name="target">アニメーションさせるターゲット</param>
        /// <param name="prop">アニメーションさせるプロパティ</param>
        public void Begin(DependencyObject target, DependencyProperty prop)
        {
            Storyboard.SetTarget(animation_, target);
            Storyboard.SetTargetProperty(animation_, new PropertyPath(prop));
            storyboard_.Children.Add(animation_);
            storyboard_.Begin();
        }

    }
}
